﻿Imports System
Imports System.Threading
Imports System.Runtime.InteropServices
Imports System.Text

Public Class TSharkTray

    Dim FlagMonitoring As Integer 'モニタリングの状態フラグ
    Dim psi As New System.Diagnostics.ProcessStartInfo() 'TSharkプロセス関連
    Dim p As System.Diagnostics.Process 'TSharkプロセス関連

    'INIファイル読み込み関連
    Private Declare Ansi Function GetPrivateProfileString Lib "KERNEL32.DLL" _
        Alias "GetPrivateProfileStringA" ( _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpApplicationName As String, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpKeyName As String, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpDefault As String, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpReturnedString As StringBuilder, ByVal nSize As Integer, _
            <MarshalAs(UnmanagedType.LPStr)> ByVal lpFileName As String _
    ) As Integer

    'INIファイル読み込み関連
    Private Function GetIniFileValue(ByVal IniAppName As String, ByVal IniKeyName As String, ByVal IniDefValue As String) As String

        Dim IniValue As StringBuilder = New StringBuilder(256)
        Dim i As Long

        i = GetPrivateProfileString(IniAppName, _
                                    IniKeyName, _
                                    IniDefValue, _
                                    IniValue, _
                                    200, _
                                    System.IO.Directory.GetCurrentDirectory & "\TSharkTray.ini" _
                                    )

        GetIniFileValue = Trim$(IniValue.ToString())

    End Function

    'TSharkTrayを終了
    Private Sub TSharkTray_FormClosed(ByVal sender As Object, ByVal e As FormClosedEventArgs) Handles Me.FormClosed

        Call TSharkStop()

    End Sub

    'TSharkTrayを開始
    Private Sub TSharkTray_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load

        Timer1.Interval = CInt(GetIniFileValue("TSharkTray", "INTERVAL", "1")) * 60000

        Call TSharkStart()

        Timer1.Enabled = True
        Timer2.Enabled = True
        Timer3.Enabled = True

    End Sub

    'タスクトレイ関連の動作
    Private Sub NotifyIcon1_MouseDoubleClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles NotifyIcon1.MouseDoubleClick

        With Me
            If .WindowState = FormWindowState.Minimized Then
                .WindowState = FormWindowState.Normal

                .Left = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Width - .Width
                .Top = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height - .Height

                .ShowInTaskbar = True
                .Visible = True
                .Activate()
            Else
                .ShowInTaskbar = False
                .Visible = False

                .WindowState = FormWindowState.Minimized
            End If
        End With

    End Sub

    '最小化ボタン
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click

        With Me
            .ShowInTaskbar = False
            .Visible = False

            .WindowState = FormWindowState.Minimized
        End With

    End Sub

    '終了ボタン
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click

        If MsgBox("TSharkTrayを終了しますか？", vbOKCancel + vbDefaultButton2, "TSharkTray") = vbOK Then
            Me.Close()
        End If

    End Sub

    'TSharkの起動
    Private Sub TSharkStart()

        Dim OptionFilter As String
        Dim OptionInterface As String
        Dim OptionExePath As String
        Dim OptionDateTime As String
        Dim OptionLogDir As String
        Dim OptionSwitches As String

        FlagMonitoring = False

        OptionDateTime = Now.ToString("yyyyMMddHHmmss")

        If System.IO.Directory.Exists(GetIniFileValue("TSharkTray", "LOGDIR", "C:\TEMP\")) = False Then
            NotifyIcon1.Visible = False

            MsgBox("設定されているログフォルダが存在しません。設定(LOGDIR)を確認してください。", vbOKOnly, "TSharkTray")
            End
        End If

        OptionLogDir = GetIniFileValue("TSharkTray", "LOGDIR", "C:\TEMP\") & Now.ToString("yyyy") _
                    & "\" & Now.ToString("MM") _
                    & "\" & Now.ToString("dd")

        If System.IO.File.Exists(GetIniFileValue("TSharkTray", "PATH", "C:\Program Files\Wireshark\tshark.exe")) = False Then
            NotifyIcon1.Visible = False

            MsgBox("設定パスにtshark.exeが存在しません。設定(PATH)を確認してください。", vbOKOnly, "TSharkTray")
            End
        End If

        OptionExePath = """" & GetIniFileValue("TSharkTray", "PATH", "C:\Program Files\Wireshark\tshark.exe") & """"

        OptionFilter = """" & GetIniFileValue("TSharkTray", "FILTER", "") & """"

        OptionInterface = """" & GetIniFileValue("TSharkTray", "INTERFACE", "") & """"

        OptionSwitches = "-i " & OptionInterface _
                        & " -f " & OptionFilter _
                        & " -w " & OptionLogDir & "\" & OptionDateTime & ".pcap"

        System.IO.Directory.CreateDirectory(OptionLogDir)

        psi.FileName = OptionExePath
        psi.Arguments = OptionSwitches
        psi.UseShellExecute = False
        psi.CreateNoWindow = True

        p = System.Diagnostics.Process.Start(psi)

        FlagMonitoring = True

    End Sub

    'TSharkの終了
    Private Sub TSharkStop()

        If FlagMonitoring = True Then
            If p.HasExited() = False Then
                FlagMonitoring = False

                p.Kill()

                p.Close()
                p.Dispose()
                p = Nothing
            End If
        End If

    End Sub

    'TSharkの再起動タイマー
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        Timer1.Enabled = False

        Call TSharkStop()

        Call TSharkStart()

        Timer1.Enabled = True

    End Sub

    'TSharkの生存確認タイマー
    Private Sub Timer2_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer2.Tick

        Timer2.Enabled = False

        If FlagMonitoring = True Then
            If p.HasExited() = True Then
                FlagMonitoring = False
                NotifyIcon1.Visible = False

                p.Close()
                p.Dispose()
                p = Nothing

                End
            End If
        End If

        Timer2.Enabled = True

    End Sub

    '何となく入れてみたGCタイマー
    Private Sub Timer3_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles Timer3.Tick

        Timer3.Enabled = False

        GC.Collect()

        Timer3.Enabled = True

    End Sub

End Class